package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.feign.OmsFeign;
import com.steerinfo.dil.mapper.*;
import com.steerinfo.dil.model.TmstruckUnloadResult;
import com.steerinfo.dil.service.ITmstruckUnloadResultService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * TmstruckUnloadResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-09 07:58
 * 类描述
 * 修订历史：
 * 日期：2021-09-09
 * 作者：generator
 * 参考：
 * 描述：TmstruckUnloadResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckUnloadResultService")
public class TmstruckUnloadResultServiceImpl implements ITmstruckUnloadResultService {

    @Autowired
    private TmstruckUnloadResultMapper tmstruckUnloadResultMapper;

    @Autowired
    private TmstruckReceiptResultServiceImpl tmstruckReceiptResultService;

    @Autowired
    private TmstruckLoadResultMapper tmstruckLoadResultMapper;

    @Autowired
    private UtilsServiceImpl utilsService;

    @Autowired
    private TmstruckMeasureCommissionServiceImpl tmstruckMeasureCommissionService;

    @Autowired
    private TmstruckWeightResultServiceImpl tmstruckWeightResultService;

    @Autowired
    OmsFeign omsFeign;

    @Autowired
    UtilsMapper utilsMapper;

    @Autowired
    TmstruckLeaveFactoryResultServiceImpl tmstruckLeaveFactoryResultService;

    @Autowired
    private RulesServiceImpl rulesService;
    /**
     * 新增卸货实绩
     * 总实绩ID、卸货点
     * @param map
     * @return
     */
    @Override
    public int addUnloadResult(Map<String, Object> map) {
        TmstruckUnloadResult tmstruckUnloadResult = new TmstruckUnloadResult();
        //生成主键ID
        BigDecimal maxId = tmstruckUnloadResultMapper.selectMaxId();
        tmstruckUnloadResult.setId(maxId);
        //插入实绩总表ID
        tmstruckUnloadResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        // 添加路段顺序号
        tmstruckUnloadResult.setSegmentSqe(DataChange.dataToBigDecimal(map.get("segmentSqe")));
        //插入卸货点 通过总实绩ID查询卸货点
        tmstruckUnloadResult.setResultUnloadPlaceId(tmstruckUnloadResultMapper.getUnloadPointId(map));
        //新增卸车实绩
        return tmstruckUnloadResultMapper.insertSelective(tmstruckUnloadResult);
    }

    /**
     * 查询所有卸货实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getUnloadResult(Map<String, Object> map) {
        Integer orderType = (Integer) map.get("orderTypee");
        switch (orderType){
            case 1:

                break;
            case 2:

                break;
            case 3:

                break;
            case 4:

                break;
            case 5:
            case 6:
            case 7:
            case 8:
            case 17:
            case 18:
                return tmstruckUnloadResultMapper.getCGUnloadResult(map); //apiId:103
            case 9:
                return tmstruckUnloadResultMapper.getImportedDomesticNzUnloadResult(map); //apiId:355
            case 10:
                return tmstruckUnloadResultMapper.getImportedDomesticNzUnload2Result(map); //apiId:356
            case 11:
                return tmstruckUnloadResultMapper.getInFactoryUnloadResult(map); //apiId:360
            case 12:
            case 13:
            case 15:
            case 16:
                return  tmstruckUnloadResultMapper.getSporadicSuppliesUnloadResult(map);//apiId:389
        }
        return null;
    }

    /**
     * 更新卸货实绩内容 扫码人员扫码后获取信息传输数据给接口
     * 1：销售订单；2：焦炭发运计划；3：水渣发运计划；4：内转到异地库；5：采购辅料订单；6：采购老区燃料订单；
     * 7：采购新区燃料订单；8：采购进口矿(万州-厂内)；9：采购内转(达州-厂内)；10：采购内转(老区-厂内)； 11：厂内内转
     *{
     *     "orderNumber": "WYSDD2021112800000765",
     *     "warehouseId": 1
     * }
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int updateUnloadResult(Map<String, Object> map) {
        int i = 0;
        map.putAll(tmstruckUnloadResultMapper.selectTotalIdByOrderNo((String) map.get("orderNumber")));
        BigDecimal orderType = DataChange.dataToBigDecimal(map.get("orderTypee"));
        i += updateUnloadResultBy(map); //更新卸货实绩
        //路段顺序号 + 1
        i += utilsService.updateOrderLineSequence(map);
        BigDecimal resultTotalId = DataChange.dataToBigDecimal(map.get("resultTotalId"));
        BigDecimal orderId = DataChange.dataToBigDecimal(map.get("orderId"));
        Map<String, Object> mesMap = new HashMap<>();
        mesMap.put("orderId", orderId);
        mesMap.put("orderType", orderType);
        switch (orderType.intValue()){
            case 1:
                break;
            case 2:
                break;
            case 3:
                break;
            case 4:
                break;
            case 5:
            case 6:
            case 7:
            case 8:
            case 9:
            case 17:
            case 18:
                //计算计皮汽车衡 (当前无法计算汽车衡)
                int placeId = rulesService.calculateWeighbridge(mesMap);
                mesMap.put("tarePlaceId", placeId);
                i += tmstruckWeightResultService.updateWeightResult(mesMap);
                i += tmstruckReceiptResultService.addReceiptResult(map); //新增完卸车实绩后新增一条收货实绩

                //计算出厂门岗 （暂时定死）
//                Integer gatepostId = 2;
                //计皮委托发送后更新出厂门岗
//                i += tmstruckLeaveFactoryResultService.updateLeaveFactoryGatepostId(resultTotalId.intValue(), gatepostId);
                break;
            case 10:
                //计算计皮汽车衡 (当前无法计算汽车衡)
                int placeId10 = rulesService.calculateWeighbridge(mesMap);
                mesMap.put("tarePlaceId", placeId10);
                i += tmstruckWeightResultService.updateWeightResult(mesMap);
                i += tmstruckReceiptResultService.addReceiptResult(map); //新增完卸车实绩后新增一条收货实绩
                break;
            case 11:
                //判断出厂路段顺序号是否为最后一个
                i += utilsService.closeOrderNormally(map);
            case 12:
                int placeId12 = rulesService.calculateWeighbridge(mesMap);
                mesMap.put("tarePlaceId", placeId12);
                i += tmstruckWeightResultService.updateWeightResult(mesMap);
                //判断是否还有物资，如果还有物资就执行发送计量委托
                materialIdIsNorWeight(map);
                break;
        }
        //卸货成功后推送消息给websocket
        utilsService.pushMesToWebsocket((String) map.get("capacityNumber"), "卸货");
        return i;
    }



    /**
     * 扫码确认卸货 更新卸货实绩
     *  运输订单号
     *  卸货时间
     *  卸货地点
     * @param map
     * @return
     */
    public int updateUnloadResultBy(Map<String, Object> map){
        TmstruckUnloadResult tmstruckUnloadResult = new TmstruckUnloadResult();
        tmstruckUnloadResult.setResultId(DataChange.dataToBigDecimal(map.get("resultId")));
        // 修改卸货点
        if(map.get("warehouseId") != null){
            tmstruckUnloadResult.setResultUnloadPlaceId(DataChange.dataToBigDecimal(map.get("warehouseId")));
        }
        //判断物资id是否为空
        if (map.get("materialId") != null){
            tmstruckUnloadResult.setMaterialId(DataChange.dataToBigDecimal(map.get("materialId")));
        }
        //获得卸货时间
        tmstruckUnloadResult.setResultEndTime(new Date());
        //扫码后确认卸货所以卸货状态码为  同意
        tmstruckUnloadResult.setUnloadStatus("同意");
        tmstruckUnloadResult.setInsertTime(new Date());
        tmstruckUnloadResult.setInsertUsername("admin");
        //新增卸车实绩内容
        return tmstruckUnloadResultMapper.updateByPrimaryKeySelective(tmstruckUnloadResult);
    }


    //内转物流更新卸车实绩
    @Override
    public int updateUnloadresultForconverted(Map<String, Object> map) {
        //获取装车实绩id
        //通过运输订单号查找装车实绩
        String orderNumber =(String) map.get("orderNumber");
        Map<String,Object> map1 = tmstruckUnloadResultMapper.selectUnloadResultIdByOrderNumber(orderNumber);
        //获取结束时间
        Integer LoadResultId=(Integer)map1.get("loadResultId");
        Date endTime=(Date)map.get("endTime");
        TmstruckUnloadResult tmstruckUnloadResult=new TmstruckUnloadResult();
        //根据主键id设置装车结束时间
        tmstruckUnloadResult.setResultId(new BigDecimal(LoadResultId));
        tmstruckUnloadResult.setResultEndTime(endTime);
        //获取装车开始时间
        Date startTime = (Date)map1.get("startTime");
        //开始时间
        long LoadstartTime = startTime.getTime();
        long LoadendTime = endTime.getTime();
        long time = LoadendTime - LoadstartTime;
        long l = time / 1000;
        //设置标准时长
        tmstruckUnloadResult.setResultDuration(new BigDecimal(l));
        int i = tmstruckUnloadResultMapper.updateByPrimaryKeySelective(tmstruckUnloadResult);
        return i;
    }

    //根据运输订单查找装车实绩id
    @Override
    public Map<String, Object> selectLoadResultIdByOrderNumber(String orderNumber) {
        return tmstruckUnloadResultMapper.selectUnloadResultIdByOrderNumber(orderNumber);
    }


    //零星物资判断物资是否需要多次计量，并发送计量委托
    @Override
    public int materialIdIsNorWeight(Map<String, Object> map) {
        //获取订单id
        String orderNumber =(String) map.get("orderNumber");
        List<Map<String, Object>> maps = tmstruckUnloadResultMapper.selectMaterialForUnload(orderNumber);
        for (Map<String,Object> map1:maps){
            BigDecimal materialId =(BigDecimal) map1.get("materialId");
            BigDecimal orderId =(BigDecimal) map1.get("orderId");
            Map<String,Object> map2=new HashMap<>();
            map2.put("materialId",materialId);
            map2.put("orderId",orderId);
            //判断物资是否被计量
            Integer integer = tmstruckUnloadResultMapper.selectUnloadMaterialId(map2);
            if (integer==0){
                //调用计量实绩接口,发送计量委托
                tmstruckMeasureCommissionService.addLXMeasureCommission(map2);
            }
        }
        return 2;
    }

    //内转物流新增卸车实绩
    @Override
    public int addUnloadresultForconverted(Map<String, Object> map) {
        TmstruckUnloadResult tmstruckUnloadResult=new TmstruckUnloadResult();
        //设置主键id
        BigDecimal maxId = tmstruckUnloadResultMapper.selectMaxId();
        tmstruckUnloadResult.setId(maxId);
        //获取开始时间
        Date startTime =(Date) map.get("startTime");
        //获取运输订单号
        String OrderNumber=map.get("OrderNumber").toString();

        //绑定总实绩id
        Map<String,Object> map2=tmstruckLoadResultMapper.selectLoadResultIdByOrderNumber(OrderNumber);
        BigDecimal  reultTotalId=(BigDecimal)map2.get("resultTotalId");
        //设置总实绩id
        tmstruckUnloadResult.setResultTotalId(reultTotalId);
        tmstruckUnloadResult.setResultStartTime(startTime);
        //获取用户id
        Integer personnelId=(Integer)map.get("personnelId");
        String personnelName= tmstruckLoadResultMapper.selectPersonNameByPersonnelId(personnelId);
        tmstruckUnloadResult.setInsertTime(new Date());
        tmstruckUnloadResult.setInsertUsername(personnelName);
        int i=tmstruckUnloadResultMapper.insertSelective(tmstruckUnloadResult);
        return i;
    }


}
